import copy
import unittest
from pathlib import Path

from click import ClickException

from streamingcli.platform.k8s.deployment_adapter import K8SDeploymentAdapter
from streamingcli.profile.profile_adapter import DeploymentMode, ScliProfile


class TestK8SProfileAdapter(unittest.TestCase):
    K8S_TEST_PROFILE = ScliProfile(
        profile_name="test_profile",
        deployment_mode=DeploymentMode.K8S_OPERATOR,
        docker_registry_url="docker_registry_url",
        k8s_namespace="test_ns",
    )
    PROJECT_NAME = "test"
    DOCKER_TAG = "latest"

    def test_init_with_valid_profile(self):
        k8s_deployment_adapter = K8SDeploymentAdapter(
            self.K8S_TEST_PROFILE, self.DOCKER_TAG, self.PROJECT_NAME
        )
        self.assertEqual(k8s_deployment_adapter.profile_data, self.K8S_TEST_PROFILE)
        self.assertEqual(k8s_deployment_adapter.project_name, self.PROJECT_NAME)
        self.assertEqual(k8s_deployment_adapter.docker_image_tag, self.DOCKER_TAG)

    def test_init_with_invalid_profile(self):
        invalid_profile = copy.deepcopy(self.K8S_TEST_PROFILE)
        invalid_profile.k8s_namespace = None
        with self.assertRaises(ClickException):
            K8SDeploymentAdapter(invalid_profile, self.DOCKER_TAG, self.PROJECT_NAME)

    def test_generating_project_template(self):
        k8s_deployment_adapter = K8SDeploymentAdapter(
            self.K8S_TEST_PROFILE, self.DOCKER_TAG, self.PROJECT_NAME
        )
        print(k8s_deployment_adapter.generate_project_template([]))
        file_path = "tests/streamingcli/resources/platform/k8s_flink_deployment.yml"
        self.assertEqual(
            k8s_deployment_adapter.generate_project_template([]),
            Path(file_path).read_text(),
        )



from typing import Optional

import click

from streamingcli.profile.profile_adapter import (
    DeploymentMode,
    ProfileAdapter,
    ScliProfile,
)


class ProfileCommand:
    @staticmethod
    def create_profile(
        profile_name: str,
        deployment_mode: str,
        ververica_url: Optional[str] = None,
        ververica_namespace: Optional[str] = None,
        ververica_deployment_target: Optional[str] = None,
        ververica_api_token: Optional[str] = None,
        docker_registry_url: Optional[str] = None,
        k8s_namespace: Optional[str] = None,
    ) -> None:
        scli_profile = ScliProfile(
            profile_name=profile_name,
            deployment_mode=DeploymentMode(deployment_mode.upper()),
            ververica_url=ververica_url,
            ververica_namespace=ververica_namespace,
            ververica_deployment_target=ververica_deployment_target,
            docker_registry_url=docker_registry_url,
            ververica_api_token=ververica_api_token,
            k8s_namespace=k8s_namespace,
        )
        ProfileAdapter.save_profile(scli_profile=scli_profile)
        click.echo(f"Scli profile {profile_name} saved")



from typing import Optional

import click
import requests
from requests.models import Response

from streamingcli.platform.deployment_adapter import DeploymentAdapter


class VervericaDeploymentAdapter(DeploymentAdapter):
    def deploy(self, deployment_yml: str) -> Optional[str]:
        response = self.post_deployment_file(deployment_yml)

        if response.status_code != 201:
            raise click.ClickException("Failed to POST deployment.yaml file")
        else:
            deployment_name = response.json()["metadata"]["name"]
            return (
                "Created deployment: "
                + f"{self.profile_data.ververica_url}/app/#/namespaces/"
                + f"{self.profile_data.ververica_namespace}/deployments/{deployment_name}"
            )

    def validate_profile_data(self) -> None:
        if self.profile_data.ververica_url is None:
            raise click.ClickException("Missing Ververica URL attribute or profile")
        if self.profile_data.ververica_namespace is None:
            raise click.ClickException(
                "Missing Ververica Namespace attribute or profile"
            )
        if self.profile_data.ververica_deployment_target is None:
            raise click.ClickException(
                "Missing Ververica Deployment Target Name attribute or profile"
            )
        if self.profile_data.ververica_api_token is None:
            raise click.ClickException(
                "Missing Ververica APIToken secret attribute or profile"
            )
        if self.profile_data.docker_registry_url is None:
            raise click.ClickException(
                "Missing Docker repository URL attribute or profile"
            )
        if self.docker_image_tag is None or len(self.docker_image_tag) == 0:
            raise click.ClickException("Missing Docker image tag attribute")

    @staticmethod
    def get_template_name() -> str:
        return "vvp_flink_deployment.yml"

    def post_deployment_file(self, deployment_file: str) -> Response:
        deployments_url = (
            f"{self.profile_data.ververica_url}/api/v1/namespaces/"
            + f"{self.profile_data.ververica_namespace}/deployments"
        )

        response = requests.post(
            url=deployments_url,
            data=deployment_file,
            headers={
                "Content-Type": "application/yaml",
                "Authorization": f"Bearer {self.profile_data.ververica_api_token}",
            },
        )
        return response



import os
from typing import Dict, Optional

import click
import docker
from docker.errors import BuildError

from ..config import (
    ADDITIONAL_DEPENDENCIES_DIR,
    DEFAULT_FLINK_APP_NAME,
    DEFAULT_NOTEBOOK_NAME,
)
from ..docker_response_reader import DockerResponseReader
from ..jupyter.jar_handler import JarHandler
from ..jupyter.notebook_converter import ConvertedNotebook, convert_notebook
from ..project.local_project_config import LocalProjectConfig, LocalProjectConfigIO
from ..project.project_type import ProjectType


class ProjectBuilder:
    @staticmethod
    def build_project(tag_name: str) -> str:
        # Load local project config
        local_project_config = LocalProjectConfigIO.load_project_config()
        client = docker.from_env()

        if local_project_config.project_type == ProjectType.JUPYTER:
            ProjectBuilder._do_convert_jupyter_notebook(local_project_config)
        image_tag = f"{local_project_config.project_name}:{tag_name}"
        click.echo(f"Building Docker image {image_tag} ...")

        try:
            (image, logs_generator) = client.images.build(path=".", tag=image_tag)
            DockerResponseReader(logs_generator).click_echo_ok_responses()
        except BuildError as err:
            build_log = "\n".join([str(log) for log in err.build_log])
            raise click.ClickException(
                "Error raised when using Docker.\n" + err.msg + "\n" + build_log
            )

        click.echo(f"Docker image {image.short_id} created with tags: {image.tags}")
        return image.tags[0]

    @staticmethod
    def convert_jupyter_notebook() -> None:
        local_project_config = LocalProjectConfigIO.load_project_config()
        if local_project_config.project_type != ProjectType.JUPYTER:
            raise RuntimeError("Couldn't convert not jupyter project")
        ProjectBuilder._do_convert_jupyter_notebook(local_project_config)
        click.echo("Jupyter notebook converted")

    @staticmethod
    def _do_convert_jupyter_notebook(local_project_config: LocalProjectConfig) -> None:
        notebook_dir = "./src"
        notebooks = [
            os.path.join(notebook_dir, _)
            for _ in os.listdir(notebook_dir)
            if _.endswith(".ipynb")
        ]
        if len(notebooks) > 1:
            raise click.ClickException(
                f"Too many notebooks in directory {notebook_dir}"
            )
        notebook_path = (
            notebooks[0]
            if len(notebooks) == 1
            else f"{notebook_dir}/{DEFAULT_NOTEBOOK_NAME}"
        )
        converted_notebook = ProjectBuilder.convert_notebook(
            notebook_path, local_project_config.secrets
        )
        ProjectBuilder.write_notebook(converted_notebook.content)
        if converted_notebook.remote_jars or converted_notebook.local_jars:
            ProjectBuilder.get_jars(
                converted_notebook, local_project_config, notebook_dir
            )

    @staticmethod
    def get_jars(
        converted_notebook: ConvertedNotebook,
        local_project_config: LocalProjectConfig,
        notebook_dir: str,
    ) -> None:
        jar_handler = JarHandler(project_root_dir=os.getcwd())
        plugin_jars_paths = jar_handler.get_classpaths_of_jars_using_plugin()
        for path in plugin_jars_paths:
            image_path = f"{ADDITIONAL_DEPENDENCIES_DIR}/{os.path.basename(path)}"
            local_project_config.add_dependency(image_path)
        for jar in converted_notebook.remote_jars:
            local_path = jar_handler.remote_copy(jar)
            image_path = f"{ADDITIONAL_DEPENDENCIES_DIR}/{os.path.basename(local_path)}"
            local_project_config.add_dependency(image_path)
        for jar in converted_notebook.local_jars:
            jar_path = jar if os.path.isabs(jar) else f"{notebook_dir}/{jar}"
            local_path = jar_handler.local_copy(jar_path)
            image_path = f"{ADDITIONAL_DEPENDENCIES_DIR}/{os.path.basename(local_path)}"
            local_project_config.add_dependency(image_path)
        LocalProjectConfigIO.update_project_config(local_project_config)

    @staticmethod
    def convert_notebook(
        notebook_path: Optional[str] = None,
        secrets_paths: Optional[Dict[str, str]] = None,
    ) -> ConvertedNotebook:
        file_path = (
            notebook_path
            if notebook_path is not None
            else f"./src/{DEFAULT_NOTEBOOK_NAME}"
        )
        return convert_notebook(file_path, secrets_paths)

    @staticmethod
    def write_notebook(notebook_content: str) -> None:
        with open(f"./src/{DEFAULT_FLINK_APP_NAME}", "w+") as script_file:
            script_file.write(notebook_content)



from typing import Optional

import click

from streamingcli.platform.ververica.deployment_target_adapter import (
    VervericaDeploymentTargetAdapter,
)
from streamingcli.profile.profile_adapter import ProfileAdapter, ScliProfile


class DeploymentTargetCommand:
    @staticmethod
    def create_deployment_target(
        kubernetes_namespace: str,
        profile_name: str,
        deployment_target_name: Optional[str] = None,
        ververica_url: Optional[str] = None,
        ververica_namespace: Optional[str] = None,
        vvp_api_token: Optional[str] = None,
        registry_url: Optional[str] = None,
    ) -> None:
        profile = ProfileAdapter.get_or_create_temporary(profile_name)

        profile = ProfileAdapter.update_profile_data(
            profile_data=profile,
            ververica_url=ververica_url,
            ververica_namespace=ververica_namespace,
            ververica_deployment_target_name=deployment_target_name,
            ververica_webtoken_secret=vvp_api_token,
            docker_registry_url=registry_url,
        )
        DeploymentTargetCommand.validate_scli_profile(profile)
        if (
            profile.ververica_url is not None
            and profile.ververica_namespace is not None
            and profile.ververica_deployment_target is not None
            and profile.ververica_api_token is not None
        ):
            VervericaDeploymentTargetAdapter.create_deployment_target(
                ververica_url=profile.ververica_url,
                ververica_namespace=profile.ververica_namespace,
                ververica_kubernetes_namespace=kubernetes_namespace,
                ververica_webtoken_secret=profile.ververica_api_token,
                ververica_deployment_target_name=profile.ververica_deployment_target,
            )
        else:
            raise click.ClickException(
                "Missing one of deployment attribute: ververica_namespace, "
                "kubernetes_namespace, ververica_api_token, ververica_deployment_target"
            )

    @staticmethod
    def validate_scli_profile(profile_data: ScliProfile) -> None:
        if profile_data.ververica_url is None:
            raise click.ClickException("Missing Ververica URL attribute")
        if profile_data.ververica_namespace is None:
            raise click.ClickException("Missing Ververica Namespace attribute")
        if profile_data.ververica_deployment_target is None:
            raise click.ClickException(
                "Missing Ververica Deployment Target Name attribute"
            )
        if profile_data.ververica_api_token is None:
            raise click.ClickException("Missing Ververica APIToken secret attribute")



class StreamingCliError(Exception):
    """Base class for all exceptions in streamingcli module"""

    message: str
    """explanation of the error"""

    def __init__(self, message: str) -> None:
        self.message = message


class FailedToOpenNotebookFile(StreamingCliError):
    """Exception raised if Jupyter Notebook file cannot be read"""

    def __init__(self, notebook_path: str) -> None:
        self.message = f"Could not open or file does not exist: {notebook_path}"



import click
import pytest

from streamingcli.main import validate_k8s_secret


class TestK8SSecretValidator:
    """Validate K8S secret for empty string"""

    def test_k8s_secret_validation_for_empty_string(self):
        test_value = None
        assert (
            validate_k8s_secret("some_context", "some_param", test_value) == test_value
        )

    """Validate K8S secret for nonempty but incorrect string"""

    def test_k8s_secret_validation_for_non_empty_incorrect_string(self):
        with pytest.raises(click.BadParameter):
            test_value = "some"
            validate_k8s_secret("some_context", "some_param", test_value)

    """Validate K8S secret for nonempty and correct string"""

    def test_k8s_secret_validation_for_non_empty_correct_string(self):
        with pytest.raises(click.BadParameter):
            test_value = "extra: secret"
            validate_k8s_secret("some_context", "some_param", test_value)



import sys
from typing import Optional

import click
from markupsafe import re

from .docker.login_command import LoginCommand
from .error import StreamingCliError
from .platform.apitoken_create_command import VervericaApiTokenCreateCommand
from .platform.apitoken_remove_command import VervericaApiTokenRemoveCommand
from .platform.deployment_target_command import DeploymentTargetCommand
from .profile.profile_adapter import DeploymentMode
from .profile.profile_command import ProfileCommand
from .project.build_command import ProjectBuilder
from .project.cicd_command import CICDInitializer
from .project.deploy_command import ProjectDeployer
from .project.jupyter.jupyter_project_factory import JupyterProjectFactory
from .project.project_type import ProjectType
from .project.publish_command import ProjectPublisher
from .project.python.python_project_factory import PythonProjectFactory


@click.group(invoke_without_command=True)
@click.pass_context
@click.version_option()
def _cli(ctx: click.Context) -> None:
    if ctx.invoked_subcommand is None:
        click.echo(ctx.get_help())
        ctx.exit()
    pass


def cli() -> None:
    try:
        _cli()
    except StreamingCliError as err:
        click.secho(err.message, file=sys.stderr, fg="red")
        sys.exit(1)


@_cli.group()
def project() -> None:
    pass


@project.command()
@click.option(
    "--project_name",
    prompt="Project name",
    help="Project name which will become Flink job name in Ververica platform",
)
@click.option(
    "--project_type",
    prompt="Project type",
    required=False,
    help="Project type",
    type=click.Choice(["python", "jupyter"]),
    default="python",
)
@click.option(
    "--template_url",
    prompt="Template URL",
    prompt_required=False,
    help="Project template url",
    required=False,
)
def project_init(
    project_name: str,
    project_type: Optional[str] = None,
    template_url: Optional[str] = None,
) -> None:
    click.echo(f"Initializing streaming project: {project_name}")
    if ProjectType(project_type) == ProjectType.PYTHON:
        PythonProjectFactory.create(project_name, template_url)
    elif ProjectType(project_type) == ProjectType.JUPYTER:
        JupyterProjectFactory.create(
            project_name=project_name, template_url=template_url
        )
    else:
        raise click.exceptions.ClickException(f"Unknown project type: {project_type}")
    click.echo(f"Project: {project_name} initialized")


@project.command()
@click.option("--docker-image-tag", help="Docker image tag", default="latest")
@click.option(
    "--docker-image-registry",
    "docker_image_registry",
    help='URL for Docker registry, i.e: "https://hub.docker.com/"',
)
@click.option(
    "--docker-image-repository",
    "docker_image_repository",
    help="Docker image repository",
)
@click.option("--profile", help="Profile name to use")
@click.option(
    "--deployment-mode",
    "deployment_mode",
    type=click.Choice([x.name for x in DeploymentMode], case_sensitive=False),
)
@click.option(
    "--vvp-url",
    "ververica_url",
    help='URL for Ververica cluster, i.e: "https://vvp.streaming-platform.example.com"',
)
@click.option("--vvp-namespace", "ververica_namespace", help="Ververica namespace")
@click.option(
    "--vvp-deployment-target",
    "ververica_deployment_target_name",
    help="Ververica deployment target name",
)
@click.option(
    "--vvp-api-token",
    "vvp_api_token",
    help="Ververica WebToken secret to make API calls",
)
@click.option("--k8s-namespace", "k8s_namespace", help="Target namespace")
@click.option(
    "--overrides-from-yaml",
    help="Path to additional deployment YAML file to merge with Ververica one",
)
def project_deploy(
    docker_image_tag: str,
    docker_image_registry: Optional[str] = None,
    docker_image_repository: Optional[str] = None,
    profile: Optional[str] = None,
    deployment_mode: Optional[DeploymentMode] = None,
    ververica_url: Optional[str] = None,
    ververica_namespace: Optional[str] = None,
    ververica_deployment_target_name: Optional[str] = None,
    vvp_api_token: Optional[str] = None,
    k8s_namespace: Optional[str] = None,
    overrides_from_yaml: Optional[str] = None,
) -> None:
    ProjectDeployer.deploy_project(
        docker_image_tag=docker_image_tag,
        docker_registry_url=docker_image_registry,
        docker_image_repository=docker_image_repository,
        profile=profile,
        deployment_mode=deployment_mode,
        ververica_url=ververica_url,
        ververica_namespace=ververica_namespace,
        ververica_deployment_target_name=ververica_deployment_target_name,
        ververica_webtoken_secret=vvp_api_token,
        k8s_namespace=k8s_namespace,
        overrides_from_yaml=overrides_from_yaml,
    )


@project.command()
@click.option("--docker-image-tag", help="Project image tag", default="latest")
def project_build(docker_image_tag: str) -> None:
    ProjectBuilder.build_project(docker_image_tag)


@project.command()
def project_convert() -> None:
    ProjectBuilder.convert_jupyter_notebook()


@project.command()
@click.option(
    "--registry-url",
    prompt="Docker registry URL",
    help='URL for Docker registry, i.e: "https://hub.docker.com/"',
)
@click.option("--docker-image-tag", help="Project image tag")
def project_publish(registry_url: str, docker_image_tag: Optional[str] = None) -> None:
    ProjectPublisher.publish(registry_url, docker_image_tag)


@_cli.group()
def platform() -> None:
    pass


@platform.group("api-token")
def api_token() -> None:
    pass


def validate_k8s_secret(ctx: click.Context, param: click.Parameter, value: str) -> str:
    if value is not None and re.match(r"^[\w\-\_]+\/[\w\-\_]+$", value) is None:
        raise click.BadParameter(message="K8s secret in incorrect format")
    else:
        return value


@api_token.command()
@click.option(
    "--vvp-url",
    "ververica_url",
    prompt="Ververica URL",
    help='URL for Ververica cluster, i.e: "https://vvp.streaming-platform.example.com"',
)
@click.option(
    "--vvp-namespace",
    "ververica_namespace",
    prompt="Ververica namespace",
    help="Ververica namespace",
)
@click.option(
    "--name",
    "apitoken_name",
    prompt="Ververica ApiToken name",
    help="Ververica ApiToken name",
)
@click.option(
    "--role",
    "apitoken_role",
    prompt="Ververica ApiToken role",
    help="Ververica ApiToken role",
)
@click.option(
    "--save-to-kubernetes-secret",
    "k8s_secret",
    callback=validate_k8s_secret,
    help="Save K8s secret in format 'namespace/secret_name' i.e. 'vvp/token",
)
def platform_apitoken_create(
    ververica_url: str,
    ververica_namespace: str,
    apitoken_name: str,
    apitoken_role: str,
    k8s_secret: Optional[str] = None,
) -> None:
    VervericaApiTokenCreateCommand.create_apitoken(
        ververica_url=ververica_url,
        ververica_namespace=ververica_namespace,
        token_name=apitoken_name,
        token_role=apitoken_role,
        k8s_secret=k8s_secret,
    )


@api_token.command()
@click.option(
    "--vvp-url",
    "ververica_url",
    prompt="Ververica URL",
    help='URL for Ververica cluster, i.e: "https://vvp.streaming-platform.example.com"',
)
@click.option(
    "--vvp-namespace",
    "ververica_namespace",
    prompt="Ververica namespace",
    help="Ververica namespace",
)
@click.option(
    "--name",
    "apitoken_name",
    prompt="Ververica ApiToken name",
    help="Ververica ApiToken name",
)
def platform_apitoken_remove(
    ververica_url: str, ververica_namespace: str, apitoken_name: str
) -> None:
    VervericaApiTokenRemoveCommand.remove_apitoken(
        ververica_url=ververica_url,
        ververica_namespace=ververica_namespace,
        token_name=apitoken_name,
    )


@platform.group()
def deployment_target() -> None:
    pass


@deployment_target.command()
@click.option(
    "--kubernetes-namespace",
    prompt="Kubernetes namespace name",
    help="Kubernetes namespace name",
    required=True,
)
@click.option("--profile", help="Profile name to use", required=True)
@click.option("--name", help="Ververica deployment target name")
@click.option(
    "--vvp-url",
    required=False,
    help='URL for Ververica cluster, i.e: "https://vvp.streaming-platform.example.com"',
)
@click.option("--vvp-namespace", required=False, help="Ververica namespace")
@click.option("--vvp-api-token", required=False, help="Ververica API Token")
@click.option(
    "--registry-url",
    prompt="Docker registry URL",
    help='URL for Docker registry, i.e: "https://hub.docker.com/"',
)
def deployment_target_create(
    kubernetes_namespace: str,
    profile: str,
    name: Optional[str] = None,
    vvp_url: Optional[str] = None,
    vvp_namespace: Optional[str] = None,
    vvp_api_token: Optional[str] = None,
    registry_url: Optional[str] = None,
) -> None:
    DeploymentTargetCommand.create_deployment_target(
        kubernetes_namespace=kubernetes_namespace,
        deployment_target_name=name,
        profile_name=profile,
        ververica_url=vvp_url,
        ververica_namespace=vvp_namespace,
        vvp_api_token=vvp_api_token,
        registry_url=registry_url,
    )


@project.group()
def cicd() -> None:
    pass


@cicd.command()
@click.option(
    "--provider",
    prompt="Provider's name",
    help="Provider's name",
    type=click.Choice(["gitlab"], case_sensitive=False),
)
def cicd_setup(provider: str) -> None:
    CICDInitializer.setup_cicd(provider)


@_cli.group()
def profile() -> None:
    pass


@profile.command()
@click.argument("profile_name")
@click.option(
    "--deployment-mode",
    "deployment_mode",
    type=click.Choice([x.name for x in DeploymentMode], case_sensitive=False),
    default="VVP",
)
@click.option(
    "--vvp-url",
    required=False,
    help='URL for Ververica cluster, i.e: "https://vvp.streaming-platform.example.com"',
)
@click.option("--vvp-namespace", required=False, help="Ververica namespace")
@click.option(
    "--vvp-deployment-target", required=False, help="Ververica deployment target name"
)
@click.option("--vvp-api-token", required=False, help="Ververica API Token")
@click.option(
    "--docker-registry-url",
    required=False,
    help='URL for Docker registry, i.e: "https://hub.docker.com/"',
)
@click.option("--k8s-namespace", "k8s_namespace", help="Target namespace")
def add_profile(
    profile_name: str,
    deployment_mode: str,
    vvp_url: Optional[str],
    vvp_namespace: Optional[str],
    vvp_deployment_target: Optional[str],
    vvp_api_token: Optional[str],
    docker_registry_url: Optional[str],
    k8s_namespace: Optional[str],
) -> None:
    ProfileCommand.create_profile(
        profile_name=profile_name,
        deployment_mode=deployment_mode,
        ververica_url=vvp_url,
        ververica_namespace=vvp_namespace,
        ververica_deployment_target=vvp_deployment_target,
        ververica_api_token=vvp_api_token,
        docker_registry_url=docker_registry_url,
        k8s_namespace=k8s_namespace,
    )


@_cli.group()
def docker() -> None:
    pass


@docker.command()
@click.option("--username", required=True, help="The registry username")
@click.option("--password", required=True, help="The plaintext password")
@click.option(
    "--docker-registry-url",
    required=True,
    help="URL to the registry. E.g. https://index.docker.io/v1/",
)
def docker_login(username: str, password: str, docker_registry_url: str) -> None:
    LoginCommand.docker_login(username, password, docker_registry_url)


project.add_command(project_init, "init")
project.add_command(project_deploy, "deploy")
project.add_command(project_convert, "convert")
project.add_command(project_build, "build")
project.add_command(project_publish, "publish")
api_token.add_command(platform_apitoken_create, "create")
api_token.add_command(platform_apitoken_remove, "remove")
deployment_target.add_command(deployment_target_create, "create")
cicd.add_command(cicd_setup, "setup")
profile.add_command(add_profile, "add")
docker.add_command(docker_login, "login")

if __name__ == "__main__":
    cli()



import click

from streamingcli.platform.ververica.apitoken_adapter import VervericaApiTokenAdapter


class VervericaApiTokenRemoveCommand:
    @staticmethod
    def remove_apitoken(
        ververica_url: str, ververica_namespace: str, token_name: str
    ) -> None:
        VervericaApiTokenAdapter.remove_token(
            ververica_url=ververica_url,
            ververica_namespace=ververica_namespace,
            apitoken_name=token_name,
        )
        click.echo("Ververica API token removed")



import json
from typing import Dict

import click
import requests


class VervericaApiTokenAdapter:
    @staticmethod
    def create_token(
        ververica_url: str,
        ververica_namespace: str,
        apitoken_name: str,
        apitoken_role: str,
    ) -> Dict[str, str]:
        apitokens_url = (
            f"{ververica_url}/apitokens/v1/namespaces/{ververica_namespace}/apitokens"
        )
        webtoken_name = f"namespaces/{ververica_namespace}/apitokens/{apitoken_name}"
        request_body = {"name": webtoken_name, "role": apitoken_role}
        response = requests.post(
            apitokens_url,
            json.dumps(request_body),
            headers={"accept": "application/json", "Content-Type": "application/json"},
        )
        if response.status_code == 200:
            webtoken_secret = response.json()["apiToken"]["secret"]

            token_object = {
                "namespace": ververica_namespace,
                "name": apitoken_name,
                "role": apitoken_role,
                "secret": webtoken_secret,
            }

            return token_object
        elif response.status_code == 409:
            raise click.ClickException(
                "Ververica ApiToken already exists! Remove it first"
            )
        else:
            raise click.ClickException(
                f"Ververica ApiToken generation error: {response.status_code}"
            )

    @staticmethod
    def remove_token(
        ververica_url: str, ververica_namespace: str, apitoken_name: str
    ) -> None:
        apitokens_url = f"{ververica_url}/apitokens/v1/namespaces/{ververica_namespace}/apitokens/{apitoken_name}"
        response = requests.delete(apitokens_url)
        if response.status_code != 200 and response.status_code != 404:
            raise click.ClickException("Cant remove Ververica ApiToken")



import json
from dataclasses import dataclass, field
from typing import Any, Dict, Generator, Optional

import click
import docker
from docker.client import DockerClient
from docker.errors import DockerException
from docker.models.images import Image

from streamingcli.project.local_project_config import LocalProjectConfigIO


@dataclass
class UserCredentials:
    username: str = field()
    password: str


class ProjectPublisher:
    @staticmethod
    def publish(
        docker_repository_url: str,
        docker_image_tag: Optional[str] = None,
        docker_credentials: Optional[UserCredentials] = None,
    ) -> None:
        local_project_config = LocalProjectConfigIO.load_project_config()

        client = docker.from_env()
        image_tag = docker_image_tag if docker_image_tag is not None else "latest"
        local_image_tag = f"{local_project_config.project_name}:{image_tag}"
        repository_name = f"{docker_repository_url}/{local_project_config.project_name}"

        if docker_credentials is not None:
            ProjectPublisher.authenticate(
                client, docker_credentials, docker_repository_url
            )

        image = ProjectPublisher.get_image(client, local_image_tag)
        if image is None:
            raise click.ClickException(
                f"No project image {local_image_tag} in local registry! "
                "Firstly build image with command 'scli project build'"
            )

        image.tag(repository_name, image_tag, force=True)
        push_gen = client.images.push(
            repository_name, image_tag, stream=True, decode=True
        )
        ProjectPublisher.show_progressbar(push_gen, local_image_tag)

    @staticmethod
    def get_image(client: DockerClient, image_tag: str) -> Optional[Image]:
        try:
            image = client.images.get(image_tag)
            return image
        except DockerException:
            return None

    @staticmethod
    def authenticate(
        client: DockerClient, credentials: UserCredentials, docker_registry_url: str
    ) -> None:
        client.login(
            credentials.username, credentials.password, registry=docker_registry_url
        )
        click.echo(f"Successfully logged in to {docker_registry_url}")

    @staticmethod
    def show_progressbar(data: Generator[Any, Any, Any], image_tag: str) -> None:
        with click.progressbar(
            label=f"Publishing docker image {image_tag}", length=100
        ) as bar:
            total = 0
            pushed = 0
            items: Dict[str, int] = {}
            for item in (json.loads(json.dumps(item)) for item in data):
                status = item.get("status")
                id = item.get("id")
                progress_detail = item.get("progressDetail")
                if item.get("errorDetail") is not None:
                    raise click.ClickException(
                        f"Failed to push to docker registry: {item.get('error')}"
                    )
                if item.get("aux") is not None:
                    bar.update(100)
                    pass
                elif status == "Pushing" and progress_detail is not None:
                    item_total = progress_detail.get("total")
                    item_current = progress_detail.get("current")
                    if (
                        item_total is None
                        or item_current is None
                        or item_current > item_total
                    ):
                        continue
                    elif id in items:
                        diff = item_current - items[id]
                        pushed += diff
                        items[id] = item_current
                    else:
                        items[id] = item_current
                        total += item_total
                        pushed += item_current
                    update_val = pushed / total * 100
                    bar.update(update_val - bar.pos)



from typing import Optional

import click
import requests
import yaml
from jinja2 import Environment

from streamingcli.project.template_loader import TemplateLoader


class VervericaDeploymentTargetAdapter:
    @staticmethod
    def get_existing_deployment_target_id_by_name(
        ververica_url: str,
        ververica_namespace: str,
        ververica_webtoken_secret: str,
        ververica_deployment_target_name: str,
    ) -> Optional[str]:
        dp_url = f"{ververica_url}/api/v1/namespaces/{ververica_namespace}/deployment-targets"
        response = requests.get(
            dp_url,
            headers={
                "accept": "application/yaml",
                "Content-Type": "application/yaml",
                "Authorization": f"Bearer {ververica_webtoken_secret}",
            },
        )
        all_deployment_targets = yaml.safe_load(response.content)
        for item in all_deployment_targets["items"]:
            if item["metadata"]["name"] == ververica_deployment_target_name:
                deployment_target_id = item["metadata"]["id"]
                click.echo(
                    f"Reused existing Ververica deployment target: {ververica_deployment_target_name}"
                    + f" with ID: {deployment_target_id}"
                )
                return deployment_target_id
        return None

    @staticmethod
    def format_template(
        ververica_kubernetes_namespace: str,
        ververica_namespace: str,
        ververica_deployment_target_name: str,
    ) -> str:
        deployment_target_template = TemplateLoader.load_project_template(
            "deployment_target.yml"
        )
        return (
            Environment()
            .from_string(deployment_target_template)
            .render(
                ververica_deployment_target_name=ververica_deployment_target_name,
                ververica_namespace=ververica_namespace,
                ververica_kubernetes_namespace=ververica_kubernetes_namespace,
            )
        )

    @staticmethod
    def create_deployment_target(
        ververica_url: str,
        ververica_namespace: str,
        ververica_kubernetes_namespace: str,
        ververica_webtoken_secret: str,
        ververica_deployment_target_name: str,
    ) -> str:
        deployment_target_id = (
            VervericaDeploymentTargetAdapter.get_existing_deployment_target_id_by_name(
                ververica_url=ververica_url,
                ververica_namespace=ververica_namespace,
                ververica_webtoken_secret=ververica_webtoken_secret,
                ververica_deployment_target_name=ververica_deployment_target_name,
            )
        )
        if deployment_target_id is not None:
            click.echo(
                f"Reused existing Ververica deployment target: {ververica_deployment_target_name}"
                + f" with ID: {deployment_target_id}"
            )
            return deployment_target_id

        deployment_target_yaml: str = VervericaDeploymentTargetAdapter.format_template(
            ververica_deployment_target_name=ververica_deployment_target_name,
            ververica_namespace=ververica_namespace,
            ververica_kubernetes_namespace=ververica_kubernetes_namespace,
        )

        dp_url = f"{ververica_url}/api/v1/namespaces/{ververica_namespace}/deployment-targets"
        response = requests.post(
            dp_url,
            deployment_target_yaml,
            headers={
                "accept": "application/yaml",
                "Content-Type": "application/yaml",
                "Authorization": f"Bearer {ververica_webtoken_secret}",
            },
        )

        if response.status_code != 201:
            raise click.ClickException(
                f"Ververica DeploymentTarget creation error: {response.text}"
            )
        deployment_target = yaml.safe_load(response.content)
        deployment_target_id = deployment_target["metadata"]["id"]

        click.echo(
            f"Ververica deployment target: {ververica_deployment_target_name} created with ID: {deployment_target_id}"
        )
        return deployment_target_id



import os
from typing import Any, Optional

import click

from streamingcli.config import PROFILE_ENV_VARIABLE_NAME
from streamingcli.platform.deployment_adapter import DeploymentAdapter
from streamingcli.platform.k8s.deployment_adapter import K8SDeploymentAdapter
from streamingcli.platform.ververica.deployment_adapter import (
    VervericaDeploymentAdapter,
)
from streamingcli.profile.profile_adapter import (
    DeploymentMode,
    ProfileAdapter,
    ScliProfile,
)
from streamingcli.project.local_project_config import LocalProjectConfigIO
from streamingcli.project.yaml_merger import YamlMerger


class ProjectDeployer:
    @staticmethod
    def get_profile_name(profile_name: Optional[str]) -> Optional[str]:
        if profile_name is not None:
            return profile_name
        else:
            return os.getenv(PROFILE_ENV_VARIABLE_NAME)

    @staticmethod
    def deploy_project(
        docker_image_tag: str,
        docker_registry_url: Optional[str] = None,
        docker_image_repository: Optional[str] = None,
        profile: Optional[str] = None,
        deployment_mode: Optional[DeploymentMode] = None,
        ververica_url: Optional[str] = None,
        ververica_namespace: Optional[str] = None,
        ververica_deployment_target_name: Optional[str] = None,
        ververica_webtoken_secret: Optional[str] = None,
        k8s_namespace: Optional[str] = None,
        overrides_from_yaml: Optional[str] = None,
    ) -> None:

        profile_name = ProjectDeployer.get_profile_name(profile_name=profile)
        if profile_name is None:
            profile_data = ScliProfile(profile_name="temporary")
        else:
            profile_data = ProfileAdapter.get_profile(
                profile_name=profile_name
            ) or ScliProfile(profile_name="temporary")

        local_project_config = LocalProjectConfigIO.load_project_config()
        project_name = docker_image_repository or local_project_config.project_name
        profile_data = ProfileAdapter.update_profile_data(
            profile_data=profile_data,
            deployment_mode=deployment_mode,
            ververica_url=ververica_url,
            ververica_namespace=ververica_namespace,
            ververica_deployment_target_name=ververica_deployment_target_name,
            ververica_webtoken_secret=ververica_webtoken_secret,
            docker_registry_url=docker_registry_url,
            k8s_namespace=k8s_namespace,
        )
        deployment_adapter = DeploymentAdapterFactory.get_adapter(
            profile_data.deployment_mode,
            profile_data=profile_data,
            docker_image_tag=docker_image_tag,
            project_name=project_name,
        )

        # Generate deployment YAML
        deployment_yml = deployment_adapter.generate_project_template(
            local_project_config.dependencies
        )

        if overrides_from_yaml:
            deployment_yml = YamlMerger.merge_two_yaml(
                deployment_yml, overrides_from_yaml
            )

        click.echo(f"Deploying streaming project: {project_name} ...")
        msg = deployment_adapter.deploy(deployment_yml)
        click.echo(msg)


class DeploymentAdapterFactory:
    @staticmethod
    def get_adapter(
        deployment_mode: Optional[DeploymentMode], **kwargs: Any
    ) -> DeploymentAdapter:
        if deployment_mode == DeploymentMode.K8S_OPERATOR:
            return K8SDeploymentAdapter(**kwargs)
        if deployment_mode == DeploymentMode.VVP:
            return VervericaDeploymentAdapter(**kwargs)
        raise click.ClickException("Unsupported deployment mode")



import hiyapyco

from streamingcli.project.yaml_merger import YamlMerger


class TestYamlMerger:
    """Test merging two YAML files"""

    def test_merge_two_yaml_files(self):
        merged = YamlMerger.merge_two_yaml(
            "tests/streamingcli/project/yaml/input_1.yml",
            "tests/streamingcli/project/yaml/input_2.yml",
        )
        expected = hiyapyco.dump(
            hiyapyco.load("tests/streamingcli/project/yaml/expected.yml")
        )

        assert merged == expected



import os
from dataclasses import asdict, dataclass, field, replace
from enum import Enum
from pathlib import Path
from typing import Any, Dict, Optional, Type

import click
from marshmallow_dataclass import class_schema
from yaml import SafeLoader, load, safe_dump

from streamingcli.config import (
    DEFAULT_PROFILE_DIR,
    DEFAULT_PROFILE_PATH,
    PROFILE_ENV_VARIABLE_NAME,
)


class DeploymentMode(Enum):
    VVP = "VVP"
    K8S_OPERATOR = "K8S_OPERATOR"


def custom_asdict_factory(data: Any) -> Dict[str, Any]:
    def convert_value(obj: Any) -> Any:
        if isinstance(obj, Enum):
            return obj.value
        return obj

    return {k: convert_value(v) for k, v in data}


@dataclass(repr=True)
class ScliProfile:
    profile_name: str
    deployment_mode: Optional[DeploymentMode] = field(default=DeploymentMode.VVP)
    ververica_url: Optional[str] = field(default=None)
    ververica_namespace: Optional[str] = field(default=None)
    ververica_deployment_target: Optional[str] = field(default=None)
    ververica_api_token: Optional[str] = field(default=None)
    docker_registry_url: Optional[str] = field(default=None)
    k8s_namespace: Optional[str] = field(default=None)


@dataclass
class ScliProfiles:
    profiles: Dict[str, ScliProfile] = field(default_factory=dict)


class ProfileAdapter:
    @staticmethod
    def save_profile(scli_profile: ScliProfile) -> None:
        os.makedirs(DEFAULT_PROFILE_DIR, exist_ok=True)
        profiles = ProfileAdapter.load_profiles()
        profiles_dict = profiles.profiles
        profiles_dict[scli_profile.profile_name] = scli_profile
        content = safe_dump(
            asdict(
                replace(profiles, profiles=profiles_dict),
                dict_factory=custom_asdict_factory,
            )
        )
        with open(DEFAULT_PROFILE_PATH, "w+") as file:
            file.write(content)

    @staticmethod
    def get_profile(profile_name: str) -> Optional[ScliProfile]:
        profiles_dict = ProfileAdapter.load_profiles().profiles
        return profiles_dict.get(profile_name)

    @staticmethod
    def get_or_create_temporary(ordered_profile_name: str) -> ScliProfile:
        profile_name = ProfileAdapter.get_profile_name(
            profile_name=ordered_profile_name
        )

        if profile_name is not None:
            profile = ProfileAdapter.get_profile(profile_name=profile_name)
            return (
                profile
                if profile is not None
                else ScliProfile(profile_name="temporary")
            )
        else:
            raise click.ClickException(
                f"Profile data not accessible for profile name: {ordered_profile_name}. Create profile first"
            )

    @staticmethod
    def load_profiles(default_profile_path: str = DEFAULT_PROFILE_PATH) -> ScliProfiles:
        profiles_path = Path(default_profile_path)

        if profiles_path.is_file() is False:
            return ScliProfiles(profiles={})

        with open(profiles_path, "r") as file:
            content = file.read()
            return ProfileAdapter.strict_load_yaml(content, ScliProfiles)

    @staticmethod
    def update_profile_data(
        profile_data: ScliProfile,
        deployment_mode: Optional[DeploymentMode] = None,
        ververica_url: Optional[str] = None,
        ververica_namespace: Optional[str] = None,
        ververica_deployment_target_name: Optional[str] = None,
        ververica_webtoken_secret: Optional[str] = None,
        docker_registry_url: Optional[str] = None,
        k8s_namespace: Optional[str] = None,
    ) -> ScliProfile:
        params = {}  # type: Dict[str, Any]
        if deployment_mode is not None:
            params["deployment_mode"] = deployment_mode
        if ververica_url is not None:
            params["ververica_url"] = ververica_url
        if ververica_namespace is not None:
            params["ververica_namespace"] = ververica_namespace
        if ververica_deployment_target_name is not None:
            params["ververica_deployment_target"] = ververica_deployment_target_name
        if ververica_webtoken_secret is not None:
            params["ververica_api_token"] = ververica_webtoken_secret
        if docker_registry_url is not None:
            params["docker_registry_url"] = docker_registry_url
        if k8s_namespace is not None:
            params["k8s_namespace"] = k8s_namespace

        non_empty_params = {
            key: value for (key, value) in params.items() if value is not None
        }

        profile_data = replace(profile_data, **non_empty_params)
        return profile_data

    @staticmethod
    def get_profile_name(profile_name: Optional[str]) -> Optional[str]:
        if profile_name is not None:
            return profile_name
        else:
            return os.getenv(PROFILE_ENV_VARIABLE_NAME)

    @staticmethod
    def strict_load_yaml(yaml: str, loaded_type: Type[Any]) -> ScliProfiles:
        schema = class_schema(loaded_type)
        return schema().load(load(yaml, Loader=SafeLoader))



import hiyapyco


class YamlMerger:
    @staticmethod
    def merge_two_yaml(input1: str, yaml_file_path: str) -> str:
        merged_result = hiyapyco.load(
            [input1, yaml_file_path],
            method=hiyapyco.METHOD_MERGE,
            interpolate=True,
            failonmissingfiles=True,
            mergelists=True,
            castinterpolated=True,
        )

        return hiyapyco.dump(merged_result)



from dataclasses import dataclass

import click
from jinja2.environment import Environment

from streamingcli.project.local_project_config import LocalProjectConfigIO
from streamingcli.project.template_loader import TemplateLoader


@dataclass
class ProviderConfig:
    templateName: str
    outputFileName: str


class CICDInitializer:
    @staticmethod
    def setup_cicd(provider: str) -> None:
        provider_config = CICDInitializer.get_providers_config(provider)
        local_project_config = LocalProjectConfigIO.load_project_config()
        project_name = local_project_config.project_name
        project_version = local_project_config.project_version
        cicd_yaml = CICDInitializer.generate_from_template(
            template_name=provider_config.templateName,
            project_name=project_name,
            project_version=project_version,
        )
        CICDInitializer.save_yaml_file(cicd_yaml, provider_config.outputFileName)
        click.echo(
            f"Initialized {provider} CICD configuration file for project: {project_name}"
        )

    @staticmethod
    def generate_from_template(
        template_name: str, project_name: str, project_version: str
    ) -> str:
        template = TemplateLoader.load_project_template(template_name)
        return (
            Environment()
            .from_string(template)
            .render(project_name=project_name, project_version=project_version)
        )

    @staticmethod
    def save_yaml_file(yaml: str, otput_file_name: str) -> None:
        with open(f"./{otput_file_name}", "w") as cicd_file:
            cicd_file.write(yaml)

    @staticmethod
    def get_providers_config(provider: str) -> ProviderConfig:
        providers_dict = {"gitlab": ProviderConfig("gitlab-ci.yml", ".gitlab-ci.yml")}
        return providers_dict[provider]



import argparse
import json
import os
import re
import shlex
import sys
from dataclasses import asdict, dataclass, field
from typing import Any, Dict, List, Optional, Sequence, Tuple, cast

import autopep8
import nbformat
import sqlparse
from jinja2 import Environment

from ..error import FailedToOpenNotebookFile, StreamingCliError
from ..project.template_loader import TemplateLoader


@dataclass
class ConvertedNotebook:
    content: str
    remote_jars: List[str] = field(default_factory=lambda: [])
    local_jars: List[str] = field(default_factory=lambda: [])


@dataclass
class NotebookEntry:
    type: str


@dataclass
class Sql(NotebookEntry):
    value: str = ""
    type: str = "SQL"


@dataclass
class RegisterUdf(NotebookEntry):
    function_name: str = ""
    object_name: str = ""
    type: str = "REGISTER_UDF"


@dataclass
class RegisterJavaUdf(RegisterUdf):
    type: str = "REGISTER_JAVA_UDF"


@dataclass
class Code(NotebookEntry):
    value: str = ""
    type: str = "CODE"


@dataclass
class RegisterJar(NotebookEntry):
    url: str = ""
    type: str = "REGISTER_JAR"


@dataclass
class RegisterLocalJar(NotebookEntry):
    local_path: str = ""
    type: str = "REGISTER_LOCAL_JAR"


class NotebookConverter:
    """
    Read Jupyter Notebook .ipynb file and convert it to Python script.
    """

    notebook_path: str
    """String representing path to .ipynb file"""

    secrets_paths: Dict[str, str]
    """Mapping from secret variable name to path of the file holding the secret."""

    hidden_variable_counter: int
    """Counter of hidden variables that are read from ENVs"""

    _udf_parser = argparse.ArgumentParser()
    """Argument parser for %register_udf magic"""
    _udf_parser.add_argument("--function_name", "-n")
    _udf_parser.add_argument("--object_name", "-u")
    _udf_parser.add_argument("--language", "-l")
    _udf_parser.add_argument("--remote_path", "-r")
    _udf_parser.add_argument("--local_path", "-p")

    _load_config_parser = argparse.ArgumentParser()
    """Argument parser for %load_config magic"""
    _load_config_parser.add_argument("--path", "-p")

    _load_secret_parser = argparse.ArgumentParser()
    """Argument parser for %load_secret magic"""
    _load_secret_parser.add_argument("--path", "-p")
    _load_secret_parser.add_argument("varname")

    _flink_execute_sql_file_parser = argparse.ArgumentParser()
    """Argument parser for %flink_execute_sql_file magic"""
    _flink_execute_sql_file_parser.add_argument("--path", "-p")

    def __init__(self, notebook_path: str, secrets_paths: Optional[Dict[str, str]]):
        self.notebook_path = notebook_path
        self.secrets_paths = secrets_paths or {}
        self.hidden_variable_counter = 0

    """
    Read and convert Jupyter Notebook

    :return: object representing converted Jupyter Notebook
    :rtype: ConvertedNotebook
    """

    def convert_notebook(self) -> ConvertedNotebook:
        try:
            notebook = self._load_notebook(self.notebook_path)
            code_cells = filter(lambda _: _.cell_type == "code", notebook.cells)
            script_entries: List[NotebookEntry] = []
            script_entries.extend(self._load_secrets_from_scli_config())
            script_entries.extend(
                self._read_init_sql(os.path.dirname(self.notebook_path))
            )
            for cell in code_cells:
                entries = self._get_notebook_entry(
                    cell, os.path.dirname(self.notebook_path)
                )
                if entries:
                    for e in entries:
                        script_entries.append(e)
            return self._render_flink_app(script_entries)
        except IOError:
            raise FailedToOpenNotebookFile(self.notebook_path)
        except Exception:  # noqa
            raise StreamingCliError(
                f"Unexpected exception while converting notebook file: {sys.exc_info()}"
            )

    @staticmethod
    def _load_notebook(notebook_path: str) -> nbformat.NotebookNode:
        with open(notebook_path, "r+") as notebook_file:
            return nbformat.reads(notebook_file.read(), as_version=4)

    def _get_notebook_entry(
        self, cell: nbformat.NotebookNode, notebook_dir: str
    ) -> Sequence[NotebookEntry]:
        if cell.source.startswith("%"):
            return self._handle_magic_cell(cell, notebook_dir)
        elif not cell.source.startswith("##"):
            return [Code(value=cell.source)]
        else:
            return []

    def _handle_magic_cell(
        self, cell: nbformat.NotebookNode, notebook_dir: str
    ) -> Sequence[NotebookEntry]:
        source = cell.source
        if source.startswith("%%flink_execute_sql"):
            sql_statement = "\n".join(source.split("\n")[1:])
            if NotebookConverter._skip_statement(sql_statement):
                return []
            return self._convert_sql_statement_to_python_instructions(sql_statement)
        if source.startswith("%flink_execute_sql_file"):
            return self._get_statements_from_file(source, notebook_dir)
        if source.startswith("%flink_register_function"):
            args = NotebookConverter._udf_parser.parse_args(shlex.split(source)[1:])
            return [
                RegisterJavaUdf(
                    function_name=args.function_name,
                    object_name=args.object_name,
                )
                if args.language == "java"
                else RegisterUdf(
                    function_name=args.function_name, object_name=args.object_name
                )
            ]
        if source.startswith("%load_config_file"):
            return [NotebookConverter._get_variables_from_file(source, notebook_dir)]
        if source.startswith("%flink_register_jar"):
            args = NotebookConverter._udf_parser.parse_args(shlex.split(source)[1:])
            return [
                RegisterJar(url=args.remote_path)
                if args.remote_path is not None
                else RegisterLocalJar(local_path=args.local_path)
            ]
        if source.startswith("%load_secret_file"):
            args = NotebookConverter._load_secret_parser.parse_args(
                shlex.split(source)[1:]
            )
            file_path = (
                args.path if os.path.isabs(args.path) else f"{notebook_dir}/{args.path}"
            )
            return [NotebookConverter._load_secret_from_file(args.varname, file_path)]
        return []

    @staticmethod
    def _get_variables_from_file(source: str, notebook_dir: str) -> Code:
        args = NotebookConverter._load_config_parser.parse_args(shlex.split(source)[1:])
        file_path = (
            args.path if os.path.isabs(args.path) else f"{notebook_dir}/{args.path}"
        )
        loaded_variables = NotebookConverter._load_config_file(path=file_path)
        variable_strings = []
        for v in loaded_variables:
            if isinstance(loaded_variables[v], str):
                variable_strings.append(f'{v}="{loaded_variables[v]}"')
            else:
                variable_strings.append(f"{v}={loaded_variables[v]}")
        all_variable_strings = "\n".join(variable_strings)
        return Code(value=f"{all_variable_strings}")

    def _get_statements_from_file(
        self, source: str, notebook_dir: str
    ) -> Sequence[NotebookEntry]:
        args = NotebookConverter._flink_execute_sql_file_parser.parse_args(
            shlex.split(source)[1:]
        )
        file_path = (
            args.path if os.path.isabs(args.path) else f"{notebook_dir}/{args.path}"
        )
        with open(file_path, "r") as f:
            instructions = [
                instruction
                for statement in sqlparse.split(f.read())
                for instruction in self._convert_sql_statement_to_python_instructions(
                    statement.rstrip(";")
                )
            ]
        return instructions

    def _read_init_sql(self, notebook_dir: str) -> Sequence[NotebookEntry]:
        file_path = f"{notebook_dir}/init.sql"
        if os.path.exists(file_path):
            with open(file_path, "r") as f:
                return [
                    instruction
                    for statement in sqlparse.split(f.read())
                    for instruction in self._convert_sql_statement_to_python_instructions(
                        statement.rstrip(";")
                    )
                ]
        return []

    @staticmethod
    def _load_config_file(path: str) -> Dict[str, Any]:
        with open(path, "r") as json_file:
            return json.load(json_file)

    @staticmethod
    def _render_flink_app(
        notebook_entries: Sequence[NotebookEntry],
    ) -> ConvertedNotebook:
        flink_app_template = TemplateLoader.load_project_template(
            "flink_app.py.template"
        )
        flink_app_script = (
            Environment()
            .from_string(flink_app_template)
            .render(notebook_entries=notebook_entries)
        )
        remote_jars = [
            asdict(entry)["url"]
            for entry in notebook_entries
            if isinstance(entry, RegisterJar)
        ]
        local_jars = [
            asdict(entry)["local_path"]
            for entry in notebook_entries
            if isinstance(entry, RegisterLocalJar)
        ]
        return ConvertedNotebook(
            content=autopep8.fix_code(flink_app_script),
            remote_jars=remote_jars,
            local_jars=local_jars,
        )

    @staticmethod
    def _skip_statement(statement: str) -> bool:
        return (
            statement.strip().lower().startswith(("select", "show", "desc", "explain"))
        )

    def _convert_sql_statement_to_python_instructions(
        self, sql_statement: str
    ) -> List[NotebookEntry]:
        sql_statement, env_var_loads = self._convert_hidden_variables_to_env_vars(
            sql_statement
        )
        return env_var_loads + [Sql(value=sql_statement)]

    def _convert_hidden_variables_to_env_vars(
        self, sql_statement: str
    ) -> Tuple[str, List[NotebookEntry]]:
        hidden_var_pattern = r"(\$\{ *[_a-zA-Z][_a-zA-Z0-9]* *\})"
        hidden_list = cast(List[str], re.findall(hidden_var_pattern, sql_statement))
        hidden_env_load_instructions: List[NotebookEntry] = (
            [Code(value="import os")] if len(hidden_list) > 0 else []
        )
        for hidden in hidden_list:
            var_name = hidden[2:-1].strip()

            hidden_variable_index = self.hidden_variable_counter
            self.hidden_variable_counter = self.hidden_variable_counter + 1

            hidden_env_variable = f"__env_var_{hidden_variable_index}__{var_name}"
            hidden_env_load_instructions.append(
                Code(value=f'{hidden_env_variable} = os.environ["{var_name}"]')
            )

            sql_statement = sql_statement.replace(
                hidden, "{" + hidden_env_variable + "}"
            )
        return sql_statement, hidden_env_load_instructions

    def _load_secrets_from_scli_config(self) -> Sequence[NotebookEntry]:
        return [
            self._load_secret_from_file(var_name, filepath)
            for var_name, filepath in self.secrets_paths.items()
        ]

    @staticmethod
    def _load_secret_from_file(var_name: str, filepath: str) -> NotebookEntry:
        return Code(
            value=f'with open("{filepath}", "r") as secret_file:\n    {var_name} = secret_file.read().rstrip()'
        )


def convert_notebook(
    notebook_path: str, secrets_paths: Optional[Dict[str, str]] = None
) -> ConvertedNotebook:
    return NotebookConverter(notebook_path, secrets_paths).convert_notebook()



import json
from typing import Optional

import click

from streamingcli.platform.k8s.secret_adapter import KubernetesSecretAdapter
from streamingcli.platform.ververica.apitoken_adapter import VervericaApiTokenAdapter


class VervericaApiTokenCreateCommand:
    @staticmethod
    def create_apitoken(
        ververica_url: str,
        ververica_namespace: str,
        token_name: str,
        token_role: str,
        k8s_secret: Optional[str] = None,
    ) -> None:
        token_data = VervericaApiTokenAdapter.create_token(
            ververica_url=ververica_url,
            ververica_namespace=ververica_namespace,
            apitoken_name=token_name,
            apitoken_role=token_role,
        )
        click.echo(json.dumps(token_data, indent=4, sort_keys=True))

        if k8s_secret is not None:
            secret_params = k8s_secret.split("/")
            secret_namespace = secret_params[0]
            secret_name = secret_params[1]

            KubernetesSecretAdapter.save_k8s_secret(
                secret_name=secret_name,
                namespace=secret_namespace,
                secret_data=token_data,
            )
            click.echo(f"Saved k8s secret {k8s_secret}")



from pathlib import Path

PROJECT_LOCAL_CONFIG_FILE_NAME = ".streaming_config.yml"
PROJECT_K8S_CONFIGMAP_KEY = "project_configmap.json"
PLATFORM_K8S_CONFIGMAP_NAME = "streaming-platform-config"
PLATFORM_K8S_CONFIGMAP_KEY = "platform_config.json"
PLATFORM_K8S_SECRET_NAME = "streaming-platform-secret"
PLATFORM_DEFAULT_DEPLOYMENT_TARGET_NAME = "default"
PROFILE_ENV_VARIABLE_NAME = "SCLI_PROFILE"
SCLI_CONFIG_DIR_NAME = ".scli"
DEFAULT_PROFILE_DIR = f"{str(Path.home())}/{SCLI_CONFIG_DIR_NAME}"
DEFAULT_PROFILE_PATH = f"{DEFAULT_PROFILE_DIR}/profiles.yml"
PYTHON_TEMPLATE_PROJECT = "PYTHON_TEMPLATE_PROJECT"
JUPYTER_TEMPLATE_PROJECT = "JUPYTER_TEMPLATE_PROJECT"
DEFAULT_FLINK_APP_NAME = "flink_app.py"
DEFAULT_NOTEBOOK_NAME = "notebook.ipynb"
ADDITIONAL_DEPENDENCIES_DIR = "/app/lib"



from typing import Optional

from kubernetes import client, config

k8s_api_client: Optional[client.CoreV1Api] = None


class KubernetesConfigLoader:
    @staticmethod
    def get_client() -> client.CoreV1Api:
        global k8s_api_client
        if k8s_api_client is None:
            try:
                config.load_incluster_config()
            except config.ConfigException:
                try:
                    config.load_kube_config()
                except config.ConfigException:
                    raise Exception("Could not configure kubernetes python client")

            k8s_api_client = client.CoreV1Api()
        return k8s_api_client



import base64
import json
from typing import Any, Dict

import click
import docker
from docker import auth
from docker.utils import config


class LoginCommand:
    @staticmethod
    def docker_login(username: str, password: str, docker_registry_url: str) -> None:
        client = docker.from_env()
        client.login(username, password, registry=docker_registry_url, reauth=True)
        click.echo(f"Successfully logged in to {docker_registry_url}")
        # it's a bit of a hack, docker-py login method does not update auth credentials in Docker config file
        LoginCommand.update_docker_auths_config(username, password, docker_registry_url)

    @staticmethod
    def update_docker_auths_config(
        username: str, password: str, docker_registry_url: str
    ) -> None:
        config_file = config.find_config_file()
        with open(config_file, "r+") as f:
            config_dict = json.loads(f.read())
            auths = config_dict.get("auths", {})
            auth_entry = LoginCommand.build_auth_entry(
                username, password, docker_registry_url
            )
            auths.update(auth_entry)
            config_dict.update({"auths": auths})
            f.seek(0)
            f.write(json.dumps(config_dict, indent=4))
            f.truncate()

    @staticmethod
    def build_auth_entry(
        username: str, password: str, docker_registry_url: str
    ) -> Dict[Any, Any]:
        registry_index = auth.resolve_index_name(docker_registry_url)
        user_pwd = f"{username}:{password}".encode("ascii")
        return {registry_index: {"auth": base64.b64encode(user_pwd).decode("utf-8")}}



from dataclasses import asdict, dataclass, field
from enum import Enum
from typing import Any, Dict, List, Tuple, Type

import click
from marshmallow_dataclass import class_schema
from yaml import SafeLoader, load, safe_dump

from streamingcli.config import PROJECT_LOCAL_CONFIG_FILE_NAME
from streamingcli.project.project_type import ProjectType


@dataclass(repr=True)
class LocalProjectConfig:
    project_name: str
    project_version: str
    project_type: ProjectType = field(metadata={"by_value": True})
    dependencies: List[str] = field(default_factory=lambda: [])
    secrets: Dict[str, str] = field(default_factory=lambda: {})

    def add_dependency(self, dependency_path: str) -> None:
        self.dependencies.append(dependency_path)
        self.dependencies = list(dict.fromkeys(self.dependencies))


def custom_asdict_factory(data: List[Tuple[str, Any]]) -> Dict[Any, Any]:
    def convert_value(obj: Any) -> Any:
        if isinstance(obj, Enum):
            return obj.value
        return obj

    return dict((k, convert_value(v)) for k, v in data)  # noqa


class LocalProjectConfigFactory:
    @staticmethod
    def from_yaml_object(config_yaml: Dict[Any, Any]) -> LocalProjectConfig:
        project_name = config_yaml["project_name"]
        project_version = config_yaml["project_version"]
        project_type = config_yaml["project_type"]
        secrets = config_yaml.get("secrets") or {}

        return LocalProjectConfig(
            project_name=project_name,
            project_version=project_version,
            project_type=project_type,
            secrets=secrets,
        )


class LocalProjectConfigIO:
    @staticmethod
    def project_config_default_path() -> str:
        return f"./{PROJECT_LOCAL_CONFIG_FILE_NAME}"

    @staticmethod
    def save_project_config(config: LocalProjectConfig) -> None:
        config_yaml = safe_dump(asdict(config, dict_factory=custom_asdict_factory))
        with open(
            f"./{config.project_name}/{PROJECT_LOCAL_CONFIG_FILE_NAME}", "w"
        ) as config_file:
            config_file.write(config_yaml)

    @staticmethod
    def update_project_config(config: LocalProjectConfig) -> None:
        config_yaml = safe_dump(asdict(config, dict_factory=custom_asdict_factory))
        with open(f"./{PROJECT_LOCAL_CONFIG_FILE_NAME}", "w") as config_file:
            config_file.seek(0)
            config_file.write(config_yaml)
            config_file.truncate()

    @staticmethod
    def load_project_config() -> LocalProjectConfig:
        config_file_path = LocalProjectConfigIO.project_config_default_path()
        try:
            with open(config_file_path, "r+") as config_file:
                content = config_file.read()
                return LocalProjectConfigIO.strict_load_yaml(
                    content, LocalProjectConfig
                )
        except Exception:  # noqa
            raise click.ClickException(
                "Current directory is not streaming project. Initialize project first"
            )

    @staticmethod
    def strict_load_yaml(yaml: str, loaded_type: Type[Any]) -> LocalProjectConfig:
        schema = class_schema(loaded_type)
        return schema().load(load(yaml, Loader=SafeLoader))



import unittest

from streamingcli.error import FailedToOpenNotebookFile
from streamingcli.jupyter.notebook_converter import convert_notebook


class TestNotebookConverter(unittest.TestCase):
    """Test converting full Jupyter Notebook with udf to Python class"""

    def test_converter(self):
        # given
        file_path = "tests/streamingcli/resources/jupyter/notebook1.ipynb"
        # expect
        converted_notebook = convert_notebook(file_path)
        assert (
            converted_notebook.content
            == '''import sys
from pyflink.table import DataTypes
from pyflink.datastream import StreamExecutionEnvironment
from pyflink.table import StreamTableEnvironment, DataTypes
from pyflink.table.udf import udf

env = StreamExecutionEnvironment.get_execution_environment()
env.set_parallelism(1)
t_env = StreamTableEnvironment.create(env)


maximum_number_of_rows = 10
some_text_variable = "some_text_value"


number_of_rows = 10


t_env.execute_sql(f"""CREATE TABLE datagen (
    id INT
) WITH (
    'connector' = 'datagen',
    'number-of-rows' = '{number_of_rows}'
)""")


@udf(result_type=DataTypes.BOOLEAN())
def filter_print(condition, message):
    with open('filename.txt', 'a+') as f:
        print(f'{message}', file=f)
    return condition


t_env.create_temporary_function("filter_print", filter_print)
'''
        )

    def test_notebook_with_remote_java_udf_conversion(self):
        # given
        file_path = "tests/streamingcli/resources/jupyter/notebook2.ipynb"
        # expect
        converted_notebook = convert_notebook(file_path)
        assert (
            converted_notebook.content
            == '''from pyflink.datastream import StreamExecutionEnvironment
from pyflink.table import StreamTableEnvironment, DataTypes
from pyflink.table.udf import udf

env = StreamExecutionEnvironment.get_execution_environment()
env.set_parallelism(1)
t_env = StreamTableEnvironment.create(env)


maximum_number_of_rows = 10
some_text_variable = "some_text_value"


t_env.create_java_temporary_function("remote_trace", "com.getindata.TraceUDF")


t_env.create_java_temporary_function("other_function", "com.getindata.Other")


t_env.execute_sql(f"""CREATE TABLE datagen (
    id INT
) WITH (
    'connector' = 'datagen',
    'number-of-rows' = '100'
)""")
'''
        )

    def test_notebook_with_local_java_udf_conversion(self):
        # given
        file_path = "tests/streamingcli/resources/jupyter/notebook3.ipynb"
        # expect
        converted_notebook = convert_notebook(file_path)
        assert (
            converted_notebook.content
            == '''from pyflink.datastream import StreamExecutionEnvironment
from pyflink.table import StreamTableEnvironment, DataTypes
from pyflink.table.udf import udf

env = StreamExecutionEnvironment.get_execution_environment()
env.set_parallelism(1)
t_env = StreamTableEnvironment.create(env)


t_env.create_java_temporary_function("local_trace", "com.getindata.TraceUDF")


t_env.execute_sql(f"""CREATE TABLE datagen (
    id INT
) WITH (
    'connector' = 'datagen',
    'number-of-rows' = '100'
)""")
'''
        )

    def test_error_raised(self):
        with self.assertRaises(FailedToOpenNotebookFile):
            convert_notebook("not/existing/path")

    def test_notebook_with_flink_execute_sql_file(self):
        # given
        file_path = "tests/streamingcli/resources/jupyter/with_init/notebook4.ipynb"

        # expect
        converted_notebook = convert_notebook(file_path)
        assert (
            converted_notebook.content
            == '''from pyflink.datastream import StreamExecutionEnvironment
from pyflink.table import StreamTableEnvironment, DataTypes
from pyflink.table.udf import udf

env = StreamExecutionEnvironment.get_execution_environment()
env.set_parallelism(1)
t_env = StreamTableEnvironment.create(env)


t_env.execute_sql(f"""CREATE TABLE datagen (
    id INT
) WITH (
      'connector' = 'datagen',
      'number-of-rows' = '{number_of_rows}'
      )""")


t_env.execute_sql(
    f"""select * from datagen WHERE remote_trace(true, 'TRACE_ME', id)""")


t_env.execute_sql(f"""select * from other""")
'''
        )

    def test_notebook_with_show_and_describe(self):
        # given
        file_path = "tests/streamingcli/resources/jupyter/notebook5.ipynb"
        # expect
        converted_notebook = convert_notebook(file_path)
        assert (
            converted_notebook.content
            == '''from pyflink.datastream import StreamExecutionEnvironment
from pyflink.table import StreamTableEnvironment, DataTypes
from pyflink.table.udf import udf

env = StreamExecutionEnvironment.get_execution_environment()
env.set_parallelism(1)
t_env = StreamTableEnvironment.create(env)


t_env.execute_sql(f"""CREATE TABLE datagen (
    id INT
) WITH (
    'connector' = 'datagen',
    'number-of-rows' = '100'
)""")
'''
        )

    def test_notebook_hidden_to_env_conversion(self):
        # given
        file_path = "tests/streamingcli/resources/jupyter/notebook_env.ipynb"
        # expect
        converted_notebook = convert_notebook(file_path)
        assert (
            converted_notebook.content
            == '''import os
from pyflink.datastream import StreamExecutionEnvironment
from pyflink.table import StreamTableEnvironment, DataTypes
from pyflink.table.udf import udf

env = StreamExecutionEnvironment.get_execution_environment()
env.set_parallelism(1)
t_env = StreamTableEnvironment.create(env)


mysql_table_name = 'datagen'


__env_var_0__MY_ENV_VARIABLE = os.environ["MY_ENV_VARIABLE"]


t_env.execute_sql(f"""CREATE TABLE datagen (
    id INT
) WITH (
    'connector' = 'datagen',
    'number-of-rows' = '{__env_var_0__MY_ENV_VARIABLE}'
)""")


__env_var_1__MYSQL_USER = os.environ["MYSQL_USER"]


__env_var_2__MYSQL_PASSWORD = os.environ["MYSQL_PASSWORD"]


t_env.execute_sql(f"""CREATE TABLE mysql (
    id INT
) WITH (
    'connector' = 'jdbc',
    'url' = 'jdbc:mysql://localhost:3306/mydatabase',
    'table-name' = '{mysql_table_name}',
    'username' = '{__env_var_1__MYSQL_USER}',
    'password' = '{__env_var_2__MYSQL_PASSWORD}'
)""")


t_env.execute_sql(f"""INSERT INTO mysql (SELECT * FROM datagen)""")
'''
        )

    def test_notebook_load_secret(self):
        # given
        file_path = "tests/streamingcli/resources/jupyter/notebook6.ipynb"
        # expect
        converted_notebook = convert_notebook(file_path)
        assert (
            converted_notebook.content
            == '''from pyflink.datastream import StreamExecutionEnvironment
from pyflink.table import StreamTableEnvironment, DataTypes
from pyflink.table.udf import udf

env = StreamExecutionEnvironment.get_execution_environment()
env.set_parallelism(1)
t_env = StreamTableEnvironment.create(env)


with open("tests/streamingcli/resources/jupyter/secret.txt", "r") as secret_file:
    certificate = secret_file.read().rstrip()


kafka_topic = 'example_topic'


t_env.execute_sql(f"""CREATE TABLE kafka (
    id INT
) WITH (
    'connector' = 'kafka',
    'topic' = '{kafka_topic}',
    'properties.bootstrap.servers' = 'localhost:9092',
    'properties.security.protocol' = 'SSL',
    'properties.group.id' = 'testGroup',
    'scan.startup.mode' = 'earliest-offset',
    'format' = 'json',
    'properties.ssl.truststore.certificates' = '{certificate}',
    'properties.ssl.truststore.type' = 'PEM'
)""")


t_env.execute_sql(f"""CREATE TABLE mysql (
    id INT
) WITH (
    'connector' = 'jdbc',
    'url' = 'jdbc:mysql://localhost:3306/mydatabase',
    'table-name' = 'table_name',
    'username' = 'username',
    'password' = 'password'
)""")


t_env.execute_sql(f"""INSERT INTO mysql (SELECT * FROM kafka)""")
'''
        )

    def test_notebook_load_config_secrets(self):
        # given
        file_path = "tests/streamingcli/resources/jupyter/notebook1.ipynb"
        # expect
        converted_notebook = convert_notebook(
            file_path, {"certificate": "/var/secrets/secret.txt"}
        )
        assert (
            converted_notebook.content
            == '''import sys
from pyflink.table import DataTypes
from pyflink.datastream import StreamExecutionEnvironment
from pyflink.table import StreamTableEnvironment, DataTypes
from pyflink.table.udf import udf

env = StreamExecutionEnvironment.get_execution_environment()
env.set_parallelism(1)
t_env = StreamTableEnvironment.create(env)


with open("/var/secrets/secret.txt", "r") as secret_file:
    certificate = secret_file.read().rstrip()


maximum_number_of_rows = 10
some_text_variable = "some_text_value"


number_of_rows = 10


t_env.execute_sql(f"""CREATE TABLE datagen (
    id INT
) WITH (
    'connector' = 'datagen',
    'number-of-rows' = '{number_of_rows}'
)""")


@udf(result_type=DataTypes.BOOLEAN())
def filter_print(condition, message):
    with open('filename.txt', 'a+') as f:
        print(f'{message}', file=f)
    return condition


t_env.create_temporary_function("filter_print", filter_print)
'''
        )



import os
from unittest import mock

from streamingcli.config import DEFAULT_PROFILE_PATH, PROFILE_ENV_VARIABLE_NAME
from streamingcli.profile.profile_adapter import (
    DeploymentMode,
    ProfileAdapter,
    ScliProfile,
)

VVP_TEST_PROFILE = ScliProfile(
    profile_name="test_profile",
    deployment_mode=DeploymentMode.VVP,
    ververica_url="ververica_url",
    ververica_namespace="ververica_namespace",
    docker_registry_url="docker_registry_url",
    ververica_api_token="some_api_token",
    ververica_deployment_target="some_deployment_target",
)


class TestProfileAdapter:
    """Test getting profile name from env variable"""

    @mock.patch.dict(os.environ, {PROFILE_ENV_VARIABLE_NAME: "test"})
    def test_get_profile_name(self):
        assert ProfileAdapter.get_profile_name(profile_name=None) == "test"
        assert ProfileAdapter.get_profile_name(profile_name="default") == "default"

    """Test creating and managing profiles"""

    def test_create_new_profile(self):
        ProfileAdapter.save_profile(VVP_TEST_PROFILE)
        assert os.path.isfile(DEFAULT_PROFILE_PATH) is True

        saved_profile = ProfileAdapter.get_profile(
            profile_name=VVP_TEST_PROFILE.profile_name
        )
        assert saved_profile.profile_name == VVP_TEST_PROFILE.profile_name
        assert saved_profile.ververica_url == VVP_TEST_PROFILE.ververica_url
        assert saved_profile.ververica_namespace == VVP_TEST_PROFILE.ververica_namespace
        assert saved_profile.docker_registry_url == VVP_TEST_PROFILE.docker_registry_url
        assert saved_profile.ververica_api_token == VVP_TEST_PROFILE.ververica_api_token
        assert saved_profile.k8s_namespace is None
        assert (
            saved_profile.ververica_deployment_target
            == VVP_TEST_PROFILE.ververica_deployment_target
        )

    """Test updating profile data"""

    def test_updating_profile_data(self):
        updated_ververica_url = "updated_ververica_url"
        updated_profile = ProfileAdapter.update_profile_data(
            VVP_TEST_PROFILE, ververica_url=updated_ververica_url
        )
        assert updated_profile.ververica_url == updated_ververica_url

    """Test creating temporary profile"""

    def test_getting_temporary_profile(self):
        ProfileAdapter.save_profile(VVP_TEST_PROFILE)
        empty_profile = ProfileAdapter.load_profiles(default_profile_path="~/aaa.json")
        assert len(empty_profile.profiles) == 0

        existing_profile = ProfileAdapter.load_profiles()
        assert len(existing_profile.profiles) > 0



import pathlib
from typing import Optional

import click
import copier

JUPYTER_TEMPLATE_PROJECT = (
    "https://github.com/getindata/streaming-cli-jupyter-template.git"
)


class JupyterProjectFactory:
    @staticmethod
    def check_if_directory_exists(project_path: str) -> bool:
        return pathlib.Path(project_path).exists()

    @staticmethod
    def create(project_name: str, template_url: Optional[str]) -> None:
        project_path = f"./{project_name}"

        if JupyterProjectFactory.check_if_directory_exists(project_path=project_path):
            raise click.ClickException("Project directory already exists!")

        template_data = {"project_name": project_name}

        copier.copy(
            src_path=(template_url or JUPYTER_TEMPLATE_PROJECT),
            dst_path=project_path,
            data=template_data,
        )



import importlib.resources as pkg_resources

from streamingcli.project import templates


class TemplateLoader:
    @staticmethod
    def load_project_template(template_name: str) -> str:
        template = pkg_resources.read_text(templates, template_name)
        return template

    @staticmethod
    def copy_binary(file_name: str, target_path: str) -> None:
        file_content = pkg_resources.read_binary(templates, file_name)
        with open(target_path, "wb") as target_file:
            target_file.write(file_content)



import os
import sys
from typing import List

from setuptools import find_packages, setup
from setuptools.command.install import install

__version__ = "1.8.0"

with open("README.md", "r") as fh:
    long_description = fh.read()


def get_requirements(filename: str) -> List[str]:
    with open(filename, "r", encoding="utf-8") as fp:
        reqs = [
            x.strip()
            for x in fp.read().splitlines()
            if not x.strip().startswith("#") and not x.strip().startswith("-i")
        ]
    return reqs


class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""

    description = "verify that the git tag matches our version"

    def run(self) -> None:
        tag = os.getenv("CI_COMMIT_TAG")

        if tag != f"v{__version__}":
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, __version__
            )
            sys.exit(info)


EXTRAS_REQUIRE = {
    "tests": [
        "pytest>=6.2.2, <7.0.0",
        "pytest-cov>=2.8.0, <3.0.0",
        "pre-commit==2.15.0",
        "tox==3.21.1",
    ]
}


setup(
    name="streamingcli",
    version=__version__,
    author="GetInData",
    author_email="office@getindata.com",
    description="Streaming platform CLI",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/getindata/streaming-cli",
    packages=find_packages(),
    python_requires=">=3.8",
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
    ],
    install_requires=get_requirements("requirements.txt"),
    extras_require=EXTRAS_REQUIRE,
    py_modules=["streamingcli"],
    entry_points={
        "console_scripts": [
            "scli = streamingcli.main:cli",
        ],
    },
    package_data={
        "streamingcli.project": ["templates/*"],
    },
    cmdclass={
        "verify": VerifyVersionCommand,
    },
)



import os
from typing import Optional

import click
import requests
import yaml
from kubernetes.config import KUBE_CONFIG_DEFAULT_LOCATION
from requests import Response

from streamingcli.platform.deployment_adapter import DeploymentAdapter


class K8SDeploymentAdapter(DeploymentAdapter):
    def deploy(self, deployment_yml: str) -> Optional[str]:
        self.load_k8s_config()

        response = self.post_deployment_file(deployment_yml)

        if response.status_code != 201:
            raise click.ClickException("Failed to POST deployment.yaml file")
        else:
            deployment_name = response.json()["metadata"]["name"]
            namespace = response.json()["metadata"]["namespace"]
            return f"Created deployment: {deployment_name} (namespace: {namespace})"

    def validate_profile_data(self) -> None:
        if self.profile_data.k8s_namespace is None:
            raise click.ClickException("Missing K8S Namespace attribute or profile")
        if self.profile_data.docker_registry_url is None:
            raise click.ClickException(
                "Missing Docker repository URL attribute or profile"
            )
        if self.docker_image_tag is None or len(self.docker_image_tag) == 0:
            raise click.ClickException("Missing Docker image tag attribute")

    @staticmethod
    def get_template_name() -> str:
        return "k8s_flink_deployment.yml"

    def post_deployment_file(self, deployment_file: str) -> Response:
        deployments_url = (
            f"{self.cluster}/apis/flink.apache.org/v1beta1/namespaces/"
            + f"{self.profile_data.k8s_namespace}/flinkdeployments"
        )

        response = requests.post(
            url=deployments_url,
            data=deployment_file,
            verify=self.ca_crt,
            cert=(self.client_crt, self.client_key),
            headers={
                "Content-Type": "application/yaml",
            },
        )
        return response

    def load_k8s_config(self) -> None:
        path = os.path.expanduser(KUBE_CONFIG_DEFAULT_LOCATION)
        with open(path, "r") as stream:
            parsed_yaml = yaml.safe_load(stream)
            current_context = parsed_yaml["current-context"]
            for cluster in parsed_yaml["clusters"]:
                if cluster["name"] == current_context:
                    self.ca_crt = cluster["cluster"]["certificate-authority"]
                    self.cluster = cluster["cluster"]["server"]
            for user in parsed_yaml["users"]:
                if user["name"] == current_context:
                    self.client_crt = user["user"]["client-certificate"]
                    self.client_key = user["user"]["client-key"]



import os.path
from shutil import copy2
from typing import List

import requests
import wget
from importlib_metadata import entry_points


class JarHandler:
    def __init__(self, project_root_dir: str):
        self.project_root_dir = project_root_dir
        self.project_jars = os.path.join(project_root_dir, "jars")
        self.create_project_jars()

    def local_copy(self, jar_path: str) -> str:
        if not os.path.exists(jar_path):
            raise ValueError(f"Path {jar_path} does not exist.")
        full_dst = copy2(jar_path, self.project_jars)
        return JarHandler.__prepend_file_protocol(full_dst)

    def create_project_jars(self) -> None:
        if not os.path.exists(self.project_jars):
            os.makedirs(self.project_jars, exist_ok=True)
        elif not os.path.isdir(self.project_jars):
            raise ValueError(
                f"{self.project_jars} is already in the project directory with incompatible type."
            )

    def get_classpaths_of_jars_using_plugin(self) -> List[str]:
        classpath = ""
        for jar_provider in entry_points(group="catalog.jars.provider"):
            provider_f = jar_provider.load()
            classpath_to_add = provider_f()
            if classpath_to_add:
                classpath = self.__extend_classpath(classpath, classpath_to_add)
        return classpath.split(";")

    @staticmethod
    def __extend_classpath(current_classpaths: str, classpath_to_add: str) -> str:
        return (
            f"{current_classpaths};{classpath_to_add}"
            if len(current_classpaths) > 0
            else classpath_to_add
        )

    @staticmethod
    def __ensure_jar_accessible(remote_path: str) -> None:
        r = requests.head(remote_path, allow_redirects=True)
        if r.status_code != 200:
            raise ValueError(f"Remote path {remote_path} is not accessible")

    @staticmethod
    def __prepend_file_protocol(path: str) -> str:
        return f"file://{path}"

    def remote_copy(self, remote_path: str) -> str:
        JarHandler.__ensure_jar_accessible(remote_path)
        # Ensure wget overwrites the file. Without removing the current file `wget`
        # by default adds `__{number}` to the name.
        filename = wget.filename_from_url(remote_path)
        local_filepath = os.path.join(self.project_jars, filename)
        if os.path.exists(local_filepath):
            os.remove(local_filepath)

        # Well, wget does not seem to know that it should output an end line after it's done.
        full_dst = wget.download(url=remote_path, out=self.project_jars)
        print()

        return JarHandler.__prepend_file_protocol(full_dst)



from base64 import b64decode, b64encode
from typing import Any, Dict, Optional

from kubernetes.client import V1ObjectMeta, V1Secret
from kubernetes.client.exceptions import ApiException

from streamingcli.platform.k8s.config_loader import KubernetesConfigLoader


class KubernetesSecretAdapter:
    @staticmethod
    def load_k8s_secret(secret_name: str, namespace: str) -> Optional[Dict[Any, Any]]:
        k8s_api_client = KubernetesConfigLoader.get_client()
        try:
            secret = k8s_api_client.read_namespaced_secret(
                name=secret_name, namespace=namespace
            )

            deserialized_secret_data = {}

            for k in secret.data.keys():
                deserialized_secret_data[k] = b64decode(secret.data[k]).decode("UTF-8")

            return deserialized_secret_data
        except ApiException as e:
            if e.status == 404:
                return None
            else:
                raise e

    @staticmethod
    def save_k8s_secret(
        secret_name: str, namespace: str, secret_data: Dict[Any, Any]
    ) -> None:
        k8s_api_client = KubernetesConfigLoader.get_client()

        serialized_secret_data = {}

        for k in secret_data.keys():
            serialized_secret_data[k] = b64encode(
                secret_data[k].encode("UTF-8")
            ).decode("UTF-8")

        secret_body = V1Secret(
            api_version="v1",
            kind="Secret",
            metadata=V1ObjectMeta(
                namespace=namespace,
                name=secret_name,
                labels={
                    "streaming": "true",
                },
            ),
            type="Opaque",
            data=serialized_secret_data,
        )

        try:
            k8s_api_client.read_namespaced_secret(name=secret_name, namespace=namespace)
            k8s_api_client.patch_namespaced_secret(
                name=secret_name, namespace=namespace, body=secret_body
            )
        except ApiException as e:
            if e.status == 404:
                k8s_api_client.create_namespaced_secret(
                    namespace=namespace, body=secret_body
                )
            else:
                raise e

    @staticmethod
    def delete_k8s_configmap(configmap_name: str, namespace: str) -> None:
        k8s_api_client = KubernetesConfigLoader.get_client()
        try:
            k8s_api_client.delete_namespaced_config_map(
                name=configmap_name, namespace=namespace
            )
        except ApiException as e:
            if e.status != 404:
                raise e



from abc import ABC, abstractmethod
from typing import List, Optional

from jinja2 import Environment

from streamingcli.profile.profile_adapter import ScliProfile
from streamingcli.project.template_loader import TemplateLoader


class DeploymentAdapter(ABC):
    def __init__(
        self, profile_data: ScliProfile, docker_image_tag: str, project_name: str
    ):
        self.profile_data = profile_data
        self.docker_image_tag = docker_image_tag
        self.project_name = project_name

        self.validate_profile_data()

    @abstractmethod
    def deploy(self, deployment_yml: str) -> Optional[str]:
        pass

    @abstractmethod
    def validate_profile_data(self) -> None:
        pass

    @staticmethod
    @abstractmethod
    def get_template_name() -> str:
        pass

    def generate_project_template(self, dependencies: List[str]) -> str:
        template = TemplateLoader.load_project_template(self.get_template_name())
        params = self.profile_data.__dict__.copy()
        params["project_name"] = self.project_name
        params["docker_image_tag"] = self.docker_image_tag
        params["dependencies"] = dependencies
        return Environment().from_string(template).render(params)
